#!/usr/bin/python
# -*- coding: utf-8 -*-
import logging
import time

from logger import Logger
from singleton import Singleton
import bottle
import json
import os


class IssuanceCard(metaclass=Singleton):
    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub):
        self.log.info('Регистрация сервиса')
        self.__updateCases()

        servicestub.route('/v1/issuancecard/cardnumber/<cardNumber>', method='POST', callback=self.issuanceCard)
        servicestub.route('/v1/issuancecard/verify/phonenumber', method='POST', callback=self.verify)
        servicestub.route('/v1/issuancecard/create/card', method='POST', callback=self.create)

    def issuanceCard(self, cardNumber=''):
        self.__updateCases()
        self.log.info("Запрос данных о карте по номеру карты: '%s'" % cardNumber)
        methodPath = '/v1/issuancecard/cardnumber/' + cardNumber

        if methodPath not in self.cases:
            methodPath = '/v1/issuancecard/cardnumber/<cardNumber>'

        return self.__getResult(methodPath)

    # получение пин-кода для создания карты
    def verify(self):
        self.__updateCases()
        methodPath = "/v1/issuancecard/verify/phonenumber"
        reqBody = json.loads(bottle.request.body.read())

        for case in self.cases[methodPath]:
            if case.get('body') == reqBody:
                time.sleep(int(case.get('sleep', 0)))
                response = case.get('response')

        return self.__getResult(methodPath, response)

    # создание карты с пин-кодом
    def create(self):
        self.__updateCases()
        methodPath = "/v1/issuancecard/create/card"
        reqBody = json.loads(bottle.request.body.read())

        for case in self.cases[methodPath]:
            if case.get('body') == reqBody:
                response = case.get('response')

        return self.__getResult(methodPath, response)

    def __getResult(self, methodPath, body=None):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        if body is None:
            body = json.dumps(self.cases[methodPath][0].get('response'))

        response = bottle.HTTPResponse(status=200, body=body)
        response.add_header("Content-Type", "application/json")
        return response

    def __updateCases(self):
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'cases.json'), 'r') as cases:
                self.cases = json.load(cases, encoding='utf-8')
        except Exception as e:
            self.log.error(str(e))


instance = IssuanceCard()
