#!/usr/bin/python3

import logging
import time

from logger import Logger
from singleton import Singleton
import bottle
import json
import os


class FindFace(metaclass=Singleton):
    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub):
        self.log.info('Регистрация сервиса FindFace')
        self.__updateCases()

        servicestub.route('/auth/login/', method='POST', callback=self.login)
        servicestub.route('/events/faces/add/', method='POST', callback=self.add)
        servicestub.route('/config.json', method='GET', callback=self.config)

    def login(self):
        self.__updateCases()
        self.log.info("Авторизация")
        methodPath = '/login'

        return self.__getResult(methodPath)

    def add(self):
        self.__updateCases()
        self.log.info("Распознавание")
        methodPath = '/events/faces/add'

        return self.__getResult(methodPath)

    def config(self):
        self.__updateCases()
        self.log.info("Получение конфигурации")
        methodPath = '/config.json'

        return self.__getResult(methodPath)

    def __getResult(self, methodPath):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        responseIndex = self.cases[methodPath].get('responseIndex', '')
        body = json.dumps(self.cases[methodPath].get('response' + responseIndex))

        time.sleep(self.cases[methodPath].get('timeout', 0))

        response = bottle.HTTPResponse(status=200, body=body)
        response.add_header("Content-Type", "application/json")
        return response

    def __updateCases(self):
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'findface.json'), 'r') as cases:
                self.cases = json.load(cases, encoding='utf-8')
        except Exception as e:
            self.log.error(str(e))


instance = FindFace()
