#!/usr/bin/python
# -*- coding: utf-8 -*-

from logger import Logger
from singleton import Singleton
import bottle
import json
import os


class OnlineCards(metaclass=Singleton):
    def __init__(self):
        self.log = Logger()
        self.cases = {}

    def registerRoutes(self, servicestub):
        self.log.info('Регистрация сервиса')
        self.__updateCases()

        servicestub.route('/onlinecards/number/<cardNumber>', method='GET', callback=self.cardByNumber)
        servicestub.route('/v1/cards/number/<cardNumber>', method='GET', callback=self.cardByNumber)
        servicestub.route('/onlinecards/phonenumber/<phoneNumber>', method='GET', callback=self.cardByPhone)
        servicestub.route('/v1/cards/phonenumber/<phoneNumber>', method='GET', callback=self.cardByPhone)
        servicestub.route('/onlinecards/phonenumbers/<phoneNumber>', method='GET', callback=self.cardsByPhone)

    def cardByNumber(self, cardNumber=''):
        self.log.info("Запрос данных о карте по номеру карты: '%s'" % cardNumber)
        methodPath = '/onlinecards/number/' + cardNumber

        if methodPath not in self.cases:
            methodPath = '/onlinecards/number/<cardNumber>'

        return self.__getResult(methodPath)

    def cardByPhone(self, phoneNumber=''):
        self.log.info("Запрос данных о карте по номеру телефона: '%s'" % phoneNumber)
        methodPath = '/onlinecards/phonenumber/' + phoneNumber

        if methodPath not in self.cases:
            methodPath = '/onlinecards/phonenumber/<phoneNumber>'

        return self.__getResult(methodPath)

    def cardsByPhone(self, phoneNumber=''):
        self.log.info("Запрос списка номеров карт по номеру телефона: '%s'" % phoneNumber)
        methodPath = '/onlinecards/phonenumbers/' + phoneNumber

        if methodPath not in self.cases:
            methodPath = '/onlinecards/phonenumbers/<phoneNumber>'

        return self.__getResult(methodPath)

    def __getResult(self, methodPath):
        if methodPath not in self.cases:
            raise bottle.HTTPError(400, 'Путь "%s" не существует' % methodPath)

        if not len(self.cases[methodPath]):
            self.log.warn('В кейсах не осталось значений')
            return None

        body = json.dumps(self.cases[methodPath][0].get('response'))
        response = bottle.HTTPResponse(status=200, body=body)
        response.add_header("Content-Type", "application/json")

        # если в кейсе нет счётчика, то возвращаем str(response)
        if 'count' not in self.cases[methodPath][0]:
            return response

        self.cases[methodPath][0]['count'] -= 1

        if self.cases[methodPath][0]['count'] == 0:
            del self.cases[methodPath][0]

        return response
    
    def __updateCases(self):
        try:
            with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'onlinecards.json'), 'r') as cases:
                self.cases = json.load(cases, encoding='utf-8')
        except Exception as e:
            self.log.error(str(e))


instance = OnlineCards()
